﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using System.IO;

namespace FileListBox
{
    /// <summary>
    /// Follow steps 1a or 1b and then 2 to use this custom control in a XAML file.
    ///
    /// Step 1a) Using this custom control in a XAML file that exists in the current project.
    /// Add this XmlNamespace attribute to the root element of the markup file where it is 
    /// to be used:
    ///
    ///     xmlns:MyNamespace="clr-namespace:FileListBox"
    ///
    ///
    /// Step 1b) Using this custom control in a XAML file that exists in a different project.
    /// Add this XmlNamespace attribute to the root element of the markup file where it is 
    /// to be used:
    ///
    ///     xmlns:MyNamespace="clr-namespace:FileListBox;assembly=FileListBox"
    ///
    /// You will also need to add a project reference from the project where the XAML file lives
    /// to this project and Rebuild to avoid compilation errors:
    ///
    ///     Right click on the target project in the Solution Explorer and
    ///     "Add Reference"->"Projects"->[Select this project]
    ///
    ///
    /// Step 2)
    /// Go ahead and use your control in the XAML file.
    ///
    ///     <MyNamespace:CustomControl1/>
    ///
    /// </summary>
    public class FileListBox : ListBox //Control
    {
        /*static FileListBox()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(FileListBox), new FrameworkPropertyMetadata(typeof(FileListBox)));
        }*/

        private FileSystemWatcher fsw;
        private DirectoryInfo di;

        public string Filtr
        {
            get
            {
                return fsw.Filter;
            }
            set
            {
                fsw.Filter = value;
            }
        }
        public bool ŚledzenieZmian
        {
            get
            {                
                return fsw.EnableRaisingEvents;
            }
            set
            {
                fsw.EnableRaisingEvents = value;
            }
        }
        public string ŚcieżkaKatalogu
        {
            get
            {
                return System.IO.Path.GetFullPath(fsw.Path);
            }
            set
            {
                if (!Directory.Exists(value))
                {
                    MessageBox.Show("Wskazany katalog nie istnieje");
                    return;
                }
                fsw.Path = value;
                if (!WczytajListePlikow()) MessageBox.Show("Zmiana katalogu nie powiodła się");
            }
        }

        public FileListBox()
        {
            fsw = new FileSystemWatcher();
            fsw.EnableRaisingEvents = false;
            fsw.Filter = "*";
            fsw.Path = String.Empty;
            fsw.IncludeSubdirectories = false;
            ŚledzenieZmian = false;

            fsw.Created += new FileSystemEventHandler(fsw_Created);
            fsw.Deleted += new FileSystemEventHandler(fsw_Deleted);
            fsw.Renamed += new RenamedEventHandler(fsw_Renamed);

            ŚcieżkaKatalogu = ".";
            ŚledzenieZmian = true;
        }

        private void fsw_Created(object sender, FileSystemEventArgs e)
        {
            this.Dispatcher.Invoke(
                System.Windows.Threading.DispatcherPriority.Normal,
                (Action)delegate()
                {
                    WczytajListePlikow();
                    RaiseZmianaPlikuEvent(RodzajZmianyPliku.Nowy, e.Name, string.Empty);
                });
        }


        private void fsw_Deleted(object sender, FileSystemEventArgs e)
        {
            this.Dispatcher.Invoke(
                System.Windows.Threading.DispatcherPriority.Normal,
                (Action) delegate()
                {
                    WczytajListePlikow();
                    RaiseZmianaPlikuEvent(RodzajZmianyPliku.Usuniety, e.Name, string.Empty);
                });
        }

        private void fsw_Renamed(object sender, RenamedEventArgs e)
        {
            this.Dispatcher.Invoke(
                System.Windows.Threading.DispatcherPriority.Normal,
                (Action) delegate()
                {
                    WczytajListePlikow(); 
                    RaiseZmianaPlikuEvent(RodzajZmianyPliku.ZmianaNazwy, e.Name, e.OldName);
                });
        }

        public enum RodzajZmianyPliku
        {
            Nowy,
            Usuniety,
            ZmianaNazwy
        }

        public class ZmianaPlikuRoutedEventArgs : RoutedEventArgs
        {
            private RodzajZmianyPliku zmiana;
            private string nazwa, poprzedniaNazwa;

            public ZmianaPlikuRoutedEventArgs(RodzajZmianyPliku zmiana, string nazwa, string poprzedniaNazwa)
            {
                this.zmiana = zmiana;
                this.nazwa = nazwa;
                this.poprzedniaNazwa = poprzedniaNazwa;
            }

            public RodzajZmianyPliku Zmiana { get { return zmiana; } }
            public string Nazwa { get { return nazwa; } }
            public string PoprzedniaNazwa  { get { return poprzedniaNazwa; } }
        }

        public delegate void ZmianaPlikuRoutedEventHandler(object sender,ZmianaPlikuRoutedEventArgs e);

        public static readonly RoutedEvent zmianaPliku = EventManager.RegisterRoutedEvent(
            "Zmiana pliku", RoutingStrategy.Bubble,
            typeof(ZmianaPlikuRoutedEventHandler), typeof(FileListBox));

        public event ZmianaPlikuRoutedEventHandler ZmianaPliku
        {
            add { AddHandler(zmianaPliku, value); }
            remove { RemoveHandler(zmianaPliku, value); }
        }

        void RaiseZmianaPlikuEvent(RodzajZmianyPliku zmiana, string nazwa, string poprzedniaNazwa)
        {
            ZmianaPlikuRoutedEventArgs e = new ZmianaPlikuRoutedEventArgs(zmiana, nazwa, poprzedniaNazwa);
            e.RoutedEvent = FileListBox.zmianaPliku;
            RaiseEvent(e);
        }

        private bool WczytajListePlikow()
        {
            this.Items.Clear();
            try
            {
                di=new DirectoryInfo(ŚcieżkaKatalogu);

                DirectoryInfo[] katalogi = di.GetDirectories();
                foreach (DirectoryInfo katalog in katalogi) this.Items.Add("[" + katalog + "]");

                FileInfo[] pliki = di.GetFiles(Filtr);
                foreach (FileInfo plik in pliki) this.Items.Add(plik.Name);

                return true;
            }
            catch(Exception exc)
            {                
                MessageBox.Show("Błąd: " + exc.Message);
                return false;
            }
        }
    }
}
